#!/bin/sh
# the next line restarts using the interpreter \
exec tclsh "$0" "$@"

set rcsId {$Id: source.tcl,v 1.6 1996/12/08 16:41:30 jfontain Exp $}

# This utility can be used to combine all sourced files making a Tcl
# program into a single file, so that the source command is no longer
# required for the program to run.

# This is most useful for creating tclets (Tcl applets) running within
# the Tcl plug-in, which, as a safe interpreter, does not allow files to
# be sourced by default (read the Tcl plugin faq for ways to allow
# sourcing files).

# type "source.tcl -h" for usage.


proc sourceCode {fileName} {
    global env sourced

    if {[info exists sourced($fileName)]} {
        # do not process the same file more than once
        return {}
    }
    set sourced($fileName) {}

    set code {}
    if {[string length $fileName]==0} {
        set file stdin
    } else {
        set file [open $fileName]
    }
    set skip 0
    while {[gets $file line]>=0} {
        if {[regexp {^[ 	]*#} $line]||$skip} {
            # skip comments and eventually following line
            set skip [string match {*\\} $line]
            continue
        }
        if {[scan $line { source %s} name]>0} {
            append code [sourceCode [subst $name]]
            continue
        }
        append code $line\n
    }
    if {[string length $fileName]>0} {
        close $file
    }
    return $code
}

if {[string match -h* [lindex $argv 0]]} {
    puts "usage: $argv0 \[-o outputFile\] \[inputFile\]"
    exit
}

if {[string compare [lindex $argv 0] -o]==0} {
    set output [open [lindex $argv 1] w]
    set index 2
} else {
    set output stdout
    set index 0
}

puts $output [sourceCode [lindex $argv $index]]
close $output
